@@ -1,7 +1,3 @@
-# coding: utf-8
-from __future__ import unicode_literals
-
-import io
 import logging
 import os
 
@@ -15,7 +11,7 @@ For full documentation visit [Docums](https://khanhduy1407.github.io/docums).
 * `docums new [dir-name]` - Create a new project.
 * `docums serve` - Start the live-reloading docs server.
 * `docums build` - Build the documentation site.
-* `docums help` - Print this help message.
+* `docums -h` - Print help message and exit.
 
 ## Project layout
 
@@ -43,7 +39,8 @@ def new(output_dir):
         os.mkdir(output_dir)
 
     log.info('Writing config file: %s', config_path)
-    io.open(config_path, 'w', encoding='utf-8').write(config_text)
+    with open(config_path, 'w', encoding='utf-8') as f:
+        f.write(config_text)
 
     if os.path.exists(index_path):
         return
@@ -51,4 +48,5 @@ def new(output_dir):
     log.info('Writing initial docs: %s', index_path)
     if not os.path.exists(docs_dir):
         os.mkdir(docs_dir)
-    io.open(index_path, 'w', encoding='utf-8').write(index_text)
+    with open(index_path, 'w', encoding='utf-8') as f:
+        f.write(index_text)
